/* Sdal's userinfo for CS 1.6
original idea/author: jugger1@blueyonder.co.uk

- changed database by storing data in files to reduce memory usage.

steam_id.txt:
LastConnect
LastIP
LastPlayername
Restricted
PlayerName
PlayerName1
PlayerName2
etc.

v.1.2: - Datei mit Namen, die nicht mit geloggt werden.
v.1.0 - 1.1: - release
*/


#include <string>
#include <admin>
#include <adminlib>

new STRING_VERSION[] = "1.21";

new g_debug=0;
new g_maxnames=4;

//database
new g_DB_Path[MAX_TEXT_LENGTH]="addons/adminmod/config/userinfo/db/";
new g_DB_Index[MAX_TEXT_LENGTH]="addons/adminmod/config/userinfo/index.txt";

//excluding names to be added automatically
#define MAX_NAMES	101;
new g_File[MAX_TEXT_LENGTH];
new g_ExcludedNames[MAX_NAMES][MAX_NAME_LENGTH];
new g_loaded;

//enable greetings to player
new g_greetings=1;

//enable name restriction that player is always automatically restricted to nickname, he used on server for the first time.
new g_name_restriction_default=0;

new g_User[MAX_PLAYERS];

//storage data of player in RAM when he has entered the game
new g_LastConnect[MAX_PLAYERS][MAX_DATA_LENGTH];
new g_LastIP[MAX_PLAYERS][MAX_NAME_LENGTH];
new g_LastName[MAX_PLAYERS][MAX_NAME_LENGTH];
new g_RestrictedName[MAX_PLAYERS][MAX_NAME_LENGTH];
new g_Names[MAX_PLAYERS][MAX_DATA_LENGTH];


public plugin_init() {
	plugin_registerinfo("SDALs plugin_userinfo","Collecting playernames, ID and IP",STRING_VERSION);
	plugin_registercmd("admin_user_info", "admin_userinfo", ACCESS_ALL, "admin_user_info <target or ID or IP>: Shows all names on server for that ID or IP.");
	plugin_registercmd("admin_user_restrict", "admin_userrestrict", ACCESS_BAN, "admin_user_restrict <target> <name>: Restrict target to <name>. No <name> removes restriction");
	plugin_registercmd("admin_user_debug", "admin_user_debug", ACCESS_CONFIG, "admin_user_debug <0/1>: shows debug infos of plugin");
	plugin_registercmd("admin_user_file", "admin_userfile", ACCESS_CONFIG, "admin_user_file <path/file.txt>: file with nicknames, which get not logged/added by this plugin automatically");
	plugin_registercmd("specmode","specmode",ACCESS_ALL);
	
	if(get_vaultdata("user_file",g_File,MAX_TEXT_LENGTH)){
		read_file();
	}
	return PLUGIN_CONTINUE;
}

read_file(){
	new Text[MAX_TEXT_LENGTH];
	new i;
	new iFileSize;
	
	if(fileexists(g_File)){
		iFileSize=filesize(g_File);
		if(iFileSize>=MAX_NAMES){
			selfmessage("[USERINFO] Sorry. To many entries in file. Increase #define MAX_NAMES in *.sma file and recompile!");
			iFileSize=MAX_NAMES-1;
		}
		for(i=1;i<=iFileSize;i++){
			readfile(g_File,g_ExcludedNames[i],i,MAX_NAME_LENGTH);
		}
		g_loaded=i;
		snprintf(Text,MAX_TEXT_LENGTH,"[USERINFO] Loaded %i entries von %s",i,g_File);
		selfmessage(Text);
	}else{
		snprintf(Text,MAX_TEXT_LENGTH,"[USERINFO] Sorry. File not found: %s",g_File);
		selfmessage(Text);
	}
}

//After player enters the server, check database
public specmode(HLCommand,HLData,HLUserName,UserIndex) {
	if(g_User[UserIndex]==0){
		new Player[MAX_NAME_LENGTH];
		new AuthID[MAX_AUTHID_LENGTH];
		new PlayerFile[MAX_TEXT_LENGTH];
		new iChar;
		
		convert_string(HLUserName,Player,MAX_NAME_LENGTH);
		
		if(!check_exclude_names(Player)){
			get_userAuthID(Player,AuthID,MAX_AUTHID_LENGTH);
		
			transform_authid(AuthID,PlayerFile);

			if(fileexists(PlayerFile)){
				db_load_data(PlayerFile,UserIndex);
				db_edit_lastdata(PlayerFile,AuthID,Player);
				if(check_restrictions(Player,UserIndex)){
					rename_player(AuthID,UserIndex);
				}
				iChar=strcount(g_Names[UserIndex],'');
				if(check_name(Player,UserIndex)>0){
					if(iChar<=g_maxnames){
						debug("specmode","Add new playername!");
						db_edit(AuthID,Player,-1);
						add_name(Player,UserIndex);
					}else{
						debug("specmode","Can't add new playername! Max. names reached");
					}
				}
			}else{
				db_create_entry(PlayerFile,AuthID,Player);
				db_load_data(PlayerFile,UserIndex);
			}
			if(g_greetings){
				greetings(AuthID);
			}
			send_admin_message(AuthID,Player);
			g_User[UserIndex]=1;
		}
	}
	return PLUGIN_CONTINUE; 
}

public plugin_info(HLOldName, HLNewName, UserIndex) {
	new OldName[MAX_NAME_LENGTH];
	new NewName[MAX_NAME_LENGTH];
	new Name[MAX_NAME_LENGTH];
	new AuthID[MAX_AUTHID_LENGTH];
	new iDead;
	new iChar;
	
	convert_string(HLNewName, NewName, MAX_NAME_LENGTH);
	convert_string(HLOldName, OldName, MAX_NAME_LENGTH);
	
	if(strcasecmp(NewName,OldName)!=0){
		debug("plugin_info","Name is different");
		if(playerinfo(UserIndex,Name,MAX_NAME_LENGTH,_,_,_,iDead,AuthID)){
			if(!iDead){
				debug("plugin_info","Player is not dead!");
				if(check_restrictions(NewName,UserIndex)){
					rename_player(AuthID,UserIndex);
					return PLUGIN_HANDLED;
				}else{
					if(!check_exclude_names(NewName)){
						if(check_name(NewName,UserIndex)>0){
							iChar=strcount(g_Names[UserIndex],'');
							if(iChar<=g_maxnames){
								debug("plugin_info","Add new playername!");
								db_edit(AuthID,NewName,-1);
								add_name(NewName,UserIndex);
							}else{
								debug("plugin_info","Can't add new playername! Max. names reached");
							}					
						}
						send_admin_message(AuthID,NewName);
					}
				}
			}
		}
	}
	return PLUGIN_CONTINUE;
}

//AdminMod Functions
public admin_userinfo(HLCommand, HLData, HLUserName, UserIndex) {
	new Data[MAX_DATA_LENGTH];
	new TargetName[MAX_NAME_LENGTH];
	new Text[MAX_TEXT_LENGTH];
	new User[MAX_NAME_LENGTH];
	new Index;
	
	convert_string(HLData, Data, MAX_DATA_LENGTH);
	convert_string(HLUserName, User, MAX_NAME_LENGTH);

	if ( check_user(Data)==1 ) {
		get_username(Data, TargetName, MAX_NAME_LENGTH);

		if ( check_immunity(TargetName)!=0 ) {
			snprintf(Text, MAX_TEXT_LENGTH, "Laf. %s is an admin. That's all private!", TargetName);
			selfmessage(Text);
		} else {
			get_userindex(TargetName,Index);
			if(Index>0){
				selfmessage("******************************************");
				selfmessage("* [SDAL-USERINFO]");
				selfmessage("******************************************");
				snprintf(Text, MAX_TEXT_LENGTH, "Current Name: %s", TargetName);
				selfmessage(Text);
				strcpy(Data,g_Names[Index],MAX_DATA_LENGTH);
				strsubst(Data,""," - ",MAX_DATA_LENGTH);
				snprintf(Text, MAX_TEXT_LENGTH, "Also known as: %s", Data);
				selfmessage(Text);
				selfmessage("******************************************");
				snprintf(Text, MAX_TEXT_LENGTH, "Restricted to: %s", g_RestrictedName[Index]);
				selfmessage(Text);
				selfmessage("******************************************");
				snprintf(Text, MAX_TEXT_LENGTH, "Last Connect: %s", g_LastConnect[Index]);
				selfmessage(Text);
				snprintf(Text, MAX_TEXT_LENGTH, "Last IP: %s", g_LastIP[Index]);
				selfmessage(Text);
				snprintf(Text, MAX_TEXT_LENGTH, "Last Name: %s", g_LastName[Index]);
				selfmessage(Text);
				selfmessage("******************************************");
			}else{
				selfmessage("Unrecognized player: ");
				selfmessage(Data);
			}
		}
	} else {
		selfmessage("Unrecognized player: ");
		selfmessage(Data);
	}
	return PLUGIN_HANDLED;
}

public admin_userrestrict(HLCommand, HLData, HLUserName, UserIndex) {
	new Data[MAX_DATA_LENGTH];
	new Player[MAX_NAME_LENGTH];
	new TargetName[MAX_NAME_LENGTH];
	new RestrictedName[MAX_NAME_LENGTH];
	new Text[MAX_TEXT_LENGTH];
	new AuthID[MAX_AUTHID_LENGTH];
	new Index;
		
	convert_string(HLData, Data, MAX_DATA_LENGTH);
	
	strsep(Data," ",Player,MAX_NAME_LENGTH,RestrictedName,MAX_NAME_LENGTH);

	if ( check_user(Player)==1 ) {
		get_username(Player, TargetName, MAX_NAME_LENGTH);
		if ( check_immunity(TargetName)!=0 ) {
			snprintf(Text, MAX_TEXT_LENGTH, "Laf. %s is an admin. That's all private!", TargetName);
			selfmessage(Text);
		} else {
			get_userindex(TargetName,Index);
			if(Index>0){
				get_userAuthID(TargetName,AuthID,MAX_AUTHID_LENGTH);
				if(strlen(RestrictedName)!=0){
					strstripquotes(RestrictedName);
					strcpy(g_RestrictedName[Index],RestrictedName,MAX_NAME_LENGTH);
					db_edit(AuthID,"1",4);
					db_edit(AuthID,RestrictedName,5);
					snprintf(Text,MAX_TEXT_LENGTH,"[USERINFO] %s will be restricted to %s!",TargetName,RestrictedName);
					selfmessage(Text);
					rename_player(AuthID,UserIndex);
				}else{
					strinit(g_RestrictedName[Index]);
					db_edit(AuthID,"0",4);
					snprintf(Text,MAX_TEXT_LENGTH,"[USERINFO] Name restriction for %s removed!",TargetName);
					selfmessage(Text);
				}
			}
		}
	} else {
		selfmessage("Unrecognized player: ");
		selfmessage(Data);
	}
	return PLUGIN_HANDLED;
}

public admin_user_debug(HLCommand, HLData, HLUserName, UserIndex) {
	new Data[MAX_DATA_LENGTH];
	convert_string(HLData, Data, MAX_DATA_LENGTH);
	
	g_debug=strtonum(Data);
	
	if(g_debug>0){
		g_debug=1;
		selfmessage("[USERINFO] Debug Mode is enabled!");
	}else{
		g_debug=0;
		selfmessage("[USERINFO] Debug Mode is disabled!");
	}
	return PLUGIN_HANDLED;
}

public admin_userfile(HLCommand, HLData, HLUserName, UserIndex) {
	new Text[MAX_TEXT_LENGTH];
	new Data[MAX_DATA_LENGTH];
	convert_string(HLData, Data, MAX_DATA_LENGTH);
	
	if(fileexists(Data)){
		set_vaultdata("user_file",Data);
		strcpy(g_File,Data,MAX_TEXT_LENGTH);
		read_file();
	}else{
		snprintf(Text,MAX_TEXT_LENGTH,"[USERINFO] File not found: %s",Data);
		selfmessage(Text);
	}			
	return PLUGIN_HANDLED;
}
	
///////////////////////
// DATABASE HANDLING //
///////////////////////

//transform authid to file
transform_authid(AuthID[],PlayerFile[]){
	strcpy(PlayerFile,AuthID,MAX_TEXT_LENGTH);
	strsubst(PlayerFile, ":", "-",MAX_TEXT_LENGTH);
	snprintf(PlayerFile,MAX_TEXT_LENGTH,"%s%s.txt",g_DB_Path,PlayerFile);
	
	debug("transform_authid",PlayerFile);
}

db_load_data(PlayerFile[],UserIndex){
	new Data[MAX_NAME_LENGTH];
	new sRestricted[2];
	new iRestricted;
	new iFileSize;
	new i;
	
	//load last connect
	readfile(PlayerFile,g_LastConnect[UserIndex],1,MAX_DATA_LENGTH);
	//load last IP
	readfile(PlayerFile,g_LastIP[UserIndex],2,MAX_NAME_LENGTH);
	//load last name
	readfile(PlayerFile,g_LastName[UserIndex],3,MAX_NAME_LENGTH);
	//load restrictions
	readfile(PlayerFile,sRestricted,4,2);
	iRestricted=strtonum(sRestricted);
	if(iRestricted){
		readfile(PlayerFile,g_RestrictedName[UserIndex],5,MAX_NAME_LENGTH);
	}
	//load names
	iFileSize=filesize(PlayerFile);
	for(i=6;i<=iFileSize;i++){
		readfile(PlayerFile,Data,i,MAX_NAME_LENGTH);
		debug("db_load_data",Data);
		strcat(g_Names[UserIndex],Data,MAX_DATA_LENGTH);
		strcat(g_Names[UserIndex],"",MAX_DATA_LENGTH);
		debug("db_load_data",g_Names[UserIndex]);
	}
	
	debug("db_load_data","data of player is loaded");
}
//write data to file

//change last connect etc.
db_edit_lastdata(PlayerFile[],AuthID[],Player[]){
	new ST[MAX_TEXT_LENGTH];
	new IP[MAX_NAME_LENGTH];
	servertime(ST, MAX_DATA_LENGTH, "%d.%m.%y-%H:%M:%S");
	get_userIP(AuthID,IP,MAX_NAME_LENGTH);
	//Line 1: Date and Time of this connect
	writefile(PlayerFile,ST,1);
		
	//Line 2: IP of this connect
	writefile(PlayerFile,IP,2);
	
	//Line 3: Playername of this connect
	writefile(PlayerFile,Player,3);
	
	debug("db_edit_lastdata","Last Date/Time and last IP/Name changed");
}

// new entry
db_create_entry(PlayerFile[],AuthID[],Player[]){
	new Data[MAX_DATA_LENGTH];
	
	//create entry in index.txt
	writefile(g_DB_Index,AuthID,-1);
	
	db_edit_lastdata(PlayerFile,AuthID,Player);
	
	//Line 4: Restriction
	snprintf(Data,MAX_DATA_LENGTH,"%i",g_name_restriction_default);
	writefile(PlayerFile,Data,4);
	
	//Line 5: Restricted Name
	writefile(PlayerFile,Player,5);
	
	//if restriction is default
	if(g_name_restriction_default){
		new UserIndex;
		get_userindex(AuthID,UserIndex);
		strcpy(g_RestrictedName[UserIndex],Player,MAX_NAME_LENGTH);
	}
	
	//Line 6: The first playername
	writefile(PlayerFile,Player,6);
	
	debug("db_create_entry","new entries made!");
}

db_edit(AuthID[],Player[],iLine){
	new PlayerFile[MAX_TEXT_LENGTH];
	transform_authid(AuthID,PlayerFile);
	writefile(PlayerFile,Player,iLine);
	debug("db_edit_name","db added/changed");
}


/////////////////////
// PLUGIN HANDLING //
/////////////////////

add_name(Player[],UserIndex){
	strcat(g_Names[UserIndex],Player,MAX_TEXT_LENGTH);
	strcat(g_Names[UserIndex],"",MAX_TEXT_LENGTH);
}

check_exclude_names(Player[]){
	new i;
	for(i=1;i<=g_loaded;i++){
		if(strcasecmp(Player,g_ExcludedNames[i])==0){
			return 1;
			break;
		}
	}
	return 0;
}

check_name(Player[],UserIndex){
	new Debug[MAX_TEXT_LENGTH];
	new Data[MAX_DATA_LENGTH];
	new Name[MAX_NAME_LENGTH];
	new i;
	new ichar;
	new found; //Zeilennummer?
	
	ichar=strcount(g_Names[UserIndex],'');
	
	debug("check_name -> g_Names =",g_Names[UserIndex]);
	
	strcpy(Data,g_Names[UserIndex],MAX_DATA_LENGTH);
	
	debug("check_name -> Data =",Data);
	
	snprintf(Debug,MAX_TEXT_LENGTH,"%i",ichar);
	debug("check_name -> ichar =",Debug);
	
	for(i=0;i<=ichar;i++){
		strsep(Data,"",Name,MAX_NAME_LENGTH,Data,MAX_DATA_LENGTH);
		debug("check_name",Name);
		if(strcmp(Player,Name)==0){
			found = 0;
			debug("check_name","nickname already exists!break loop.");
			break;
		}else{
			debug("check_name","nickname does not exist!");
			found = i+1;
		}
	}
	return found;
}

check_restrictions(Player[],UserIndex){
	if(strlen(g_RestrictedName[UserIndex])!=0){
		debug("check_restrictions","restriction found!");
		if(strcmp(Player,g_RestrictedName[UserIndex])!=0){
			debug("check_restrictions","rename_player is needed!");
			return 1;
		}else{
			debug("check_restrictions","restricted name = player name. fine!");
		}
	}else{
		debug("check_restrictions","no restriction found for player!");
	}
	return 0;
}

rename_player(AuthID[],UserIndex){
	new Data[MAX_DATA_LENGTH];
	snprintf(Data,MAX_DATA_LENGTH,"name ^"%s^"",g_RestrictedName[UserIndex]);
	execclient(AuthID,Data);
	debug("rename_player","execclient executed");
}

greetings(AuthID[]){
	new Text[MAX_TEXT_LENGTH];
	new Player[MAX_NAME_LENGTH];
	get_username(AuthID,Player,MAX_NAME_LENGTH);
	snprintf(Text,MAX_TEXT_LENGTH,"Welcome,^n%s",Player);
	messageex(Player, Text, print_tty);
	
	debug("greetings",AuthID);
}

send_admin_message(AuthID[],NewName[]){
	new Text[MAX_TEXT_LENGTH];
	new Data[MAX_DATA_LENGTH];
	new Name[MAX_NAME_LENGTH];
	new i;
	new maxplayers=maxplayercount();
	new UserIndex;
	
	get_userindex(AuthID,UserIndex);
	get_username(AuthID,Name,MAX_NAME_LENGTH);
	
	debug("send_admin_message",NewName);
	
	
	strcpy(Data,g_Names[UserIndex],MAX_DATA_LENGTH);
	strsubst(Data,""," - ",MAX_DATA_LENGTH);
	snprintf(Text,MAX_TEXT_LENGTH,"[USERINFO] %s: %s",NewName,Data);
	
	debug("send_admin_message",Data);
	
	for(i=1;i<=maxplayers;i++){
		if(playerinfo(i,Name,MAX_NAME_LENGTH)){
			if(auth(Name)){
				debug("send_admin_message to",Name);
				messageex(Name,Text,print_chat);
			}
		}
	}
}

debug(Function[],Data[]){
	if (g_debug){
		new Text[MAX_TEXT_LENGTH];
		snprintf(Text,MAX_TEXT_LENGTH,"[USERINFO-DEBUG] %s -- %s",Function,Data);
		selfmessage(Text);
	}
}
	


	
	



